<footer class="footer">
    <div class="footer-right">
        <p>&copy; 2024 Data Platform HQ Authors</p>
    </div>
    <div class="footer-right">
        <a href="http://github.com/data-platform-hq/" ><i class="fab fa-github"></i> Follow us on GitHub</a>
    </div>
</footer>
