<head>
    <title>{{ title }}</title>
    <link rel="stylesheet" href="{{ url_for('oidc_static', filename='style.css') }}">
    <link rel="shortcut icon" href="{{ url_for('oidc_static', filename='favicon.ico') }}">
    <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/5.15.3/css/all.min.css">
</head>
